'$INCLUDE: 'C:\QB45\QB.BI'
'$INCLUDE: 'DOSFUNC.BI'

DIM SHARED inregs AS RegTypeX
DIM SHARED outregs AS RegTypeX

'************************************************************************
'
'    Function:  -   AllocMem%
'
' Description:  -   This function allocates conventional memory.
'
'       Entry:  -   (I) Filename
'
'     Returns:  -   (L) Segment of allocated memory.
'
' Error Codes:  -   ( 0) - No Error
'                   (-1) - DOS Error Occured
'
'************************************************************************
FUNCTION AllocMem% (BuffSize AS INTEGER)
	 inregs.ax = &H4800
rem     inregs.bx = BuffSize
     inregs.bx = 4096

PRINT "Paragraphs to allocate: ";hex$(inregs.bx)

     CALL INTERRUPTX(&H21, inregs, outregs)
	 IF (outregs.flags AND 1) = 1 THEN
		AllocMem% = -1
		DOSError = outregs.ax
	 ELSE
		AllocMem% = outregs.ax
	 END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   CloseFile%
'
' Description:  -   This function closes file opened with OpenFile%.
'
'       Entry:  -   (I) FileHandle
'
'     Returns:  -
'
' Error Codes:  -   (-1) - DOS Error Occured
'
'************************************************************************
FUNCTION CloseFile% (FileHandle AS INTEGER)
	 inregs.ax = &H3E00
	 inregs.bx = FileHandle
	 CALL INTERRUPTX(&H21, inregs, outregs)
	 IF (outregs.flags AND 1) = 1 THEN
		CloseFile% = -1
		DOSError = outregs.bx
	 ELSE
		CloseFile% = 0
	 END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   FileSize
'
' Description:  -   This function return size of a file.
'
'       Entry:  -   (S) Filename
'
'     Returns:  -   (L) FileSize
'
' Error Codes:  -   (-1) - file not found
'
'************************************************************************
FUNCTION FileSize% (lpszFileName AS STRING)
	 DIM DiskXferAddx AS DTA
	 CALL SetDTA(DiskXferAddx)

	 inregs.ax = &H4E00
	 inregs.ds = VARSEG(lpszFileName)
	 inregs.dx = SADD(lpszFileName)
	 inregs.cx = &H10
	 CALL INTERRUPTX(&H21, inregs, outregs)
	 IF (outregs.flags AND 1) = 1 THEN
		FileSize% = -1
		DOSError = outregs.ax
	 ELSE
		FileSize% = DiskXferAddx.FSize
	 END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   FreeMem
'
' Description:  -   This function frees memory allocated with AllocMem.
'
'       Entry:  -   (I) Buffer Segment
'
'     Returns:  -
'
' Error Codes:  -   ( 0) - No Error
'                   (-1) - DOS Error Occured
'
'************************************************************************
FUNCTION FreeMem% (BuffSeg AS INTEGER)
	 inregs.ax = &H4900
	 inregs.es = BuffSeg
	 CALL INTERRUPTX(&H21, inregs, outregs)
	 IF (outregs.flags AND 1) = 1 THEN
		FreeMem% = -1
		DOSError = outregs.ax
	 ELSE
		FreeMem% = 0
	 END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   MemFree
'
' Description:  -   This check the amount of free conventional memory.
'
'       Entry:  -   (I) Filename
'
'     Returns:  -   (L) Segment of allocated memory.
'
' Error Codes:  -   ( 0) - No Error
'                   (-1) - DOS Error Occured
'
'************************************************************************
FUNCTION MemFree%
	inregs.ax = &H4800
	inregs.bx = &HFFFF
	CALL INTERRUPTX(&H21, inregs, outregs)
    MemFree% = outregs.bx
	DOSError = outregs.ax
END FUNCTION

'************************************************************************
'
'    Function:  -   OpenFile%
'
' Description:  -   This function opens a file in binary mode.
'
'       Entry:  -   (S) Filename
'
'     Returns:  -   (I) FileHandle
'
' Error Codes:  -   (-1) - file not found
'
'************************************************************************
FUNCTION OpenFile% (lpszFileName AS STRING)
	 inregs.ax = &H3D00
	 inregs.ds = VARSEG(lpszFileName)
	 inregs.dx = SADD(lpszFileName)
	 CALL INTERRUPTX(&H21, inregs, outregs)
     IF (outregs.flags AND 1) = 1 THEN
		OpenFile% = -1
		DOSError = outregs.ax
	 ELSE
		OpenFile% = outregs.ax
	 END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   ReadFile%
'
' Description:  -   This function reads from file opened with OpenFile%
'
'       Entry:  -   (I) FileHandle
'                   (I) Buffer Segment
'                   (I) Number Of Bytes To Read
'
'     Returns:  -
'
' Error Codes:  -   (-1) - file not found
'
'************************************************************************
FUNCTION ReadFile% (FileHandle AS INTEGER, BuffSeg AS INTEGER, BuffSize AS INTEGER)
	 inregs.ax = &H3F00
	 inregs.bx = FileHandle
	 inregs.ds = BuffSeg
	 inregs.dx = 0
	 inregs.cx = BuffSize
     CALL INTERRUPTX(&H21, inregs, outregs)
	 IF (outregs.flags AND 1) = 1 THEN
		ReadFile% = -1
		DOSError = outregs.ax
	 ELSE
		ReadFile% = outregs.ax
	 END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   SetDTA
'
' Description:  -   This function sets the disk transfer address.
'
'       Entry:  -   (L) Seg:Off of DTA
'
'     Returns:  -
'
' Error Codes:  -   (-1) - file not found
'
'************************************************************************
SUB SetDTA (DiskXferAddx AS DTA)
	 inregs.ax = &H1A00
	 inregs.ds = VARSEG(DiskXferAddx)
	 inregs.dx = VARPTR(DiskXferAddx)
	 CALL INTERRUPTX(&H21, inregs, outregs)
END SUB

