'$INCLUDE: 'SBSMAIN.BI'

DIM SHARED inregs AS RegTypeX
DIM SHARED outregs AS RegTypeX

DIM SHARED sbSimErrorMsg$(30)


	DATA "SBSIM", 259
	DATA "No Error"
	DATA "SBSIM Currently in use"
	DATA "Bad Driver Specified"
	DATA "Bad Function Specified"
	DATA "Voice process already active"
	DATA "Couldn't start CT-VOICE"
	DATA "Couldn't start CTVDSK"
	DATA "Invalid SBSIM handle"
	DATA "Buffer not initialized"
	DATA "Bad filename for load"
	DATA "Bad file handle"
	DATA "Driver not started"
	DATA "XMS driver not installed"
	DATA "No free SBSIM handles"
	DATA "Bad file type specified"
	DATA "Couldn't free XMS block"
	DATA "Invalid source selected"
	DATA "Get-pan-position failed"
	DATA "Set-pan-position failed"
	DATA "Set-volume failed"
	DATA "Couldn't start fade/pan"
	DATA "Couldn't stop fade/pan"
	DATA "Couldn't pause fade/pan"
	DATA "Not a fade/pan operation"
	DATA "Bad mode for fade/pan"
	DATA "Couldn't start fade/pan"
	DATA "Source not fading/panning"
	DATA "FM or MIDI already playing"
	DATA "Bad MIDI mapper format"

'************************************************************************
'
'    Function:  -   FindDrv
'
' Description:  -
'
'       Entry:  -
'
'     Returns:  -
'
' Error Codes:  -
'
'************************************************************************
FUNCTION FindDrv% (IDStr AS STRING, IDOff AS INTEGER)
	DIM IntVec AS INTEGER
	DIM Segment AS LONG
	DIM Offset AS LONG
	FOR IntVec = &H80 TO &HC0
		DEF SEG = 0
		Segment = PEEK(IntVec * 4 + 3)
		Segment = Segment * 256
		Segment = Segment + PEEK(IntVec * 4 + 2)
		Offset = PEEK(IntVec * 4)
		Offset = Offset * 256
		Offset = Offset + PEEK(IntVec * 4 + 1)
	   
		DEF SEG = Segment
		strmatch = 1
		strindex = 0
		FOR strindex = 0 TO LEN(IDStr)
			IF PEEK(strindex + IDOff) <> ASC(MID$(IDStr, strindex + 1, 1) + CHR$(0)) THEN
				strmatch = 0
				EXIT FOR
			END IF
		NEXT
		IF strmatch = 1 THEN EXIT FOR
	NEXT
	DEF SEG
	IF strmatch = 1 THEN
		FindDrv% = IntVec
	ELSE
		FindDrv% = -1
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   FreeExtMem%
'
' Description:  -   This function frees an XMS handle associated with an
'                   SBSIM handle.
'
'       Entry:  -   (I) SBSIM Handle
'
'     Returns:  -    0 = no error
'                   -1 = error condition in sbSim.Status
'
' Error Codes:  -   (0, 7, 12, 15)
'                   See sbSimStatus error codes for description
'
'************************************************************************
FUNCTION FreeExtMem% (sbSim AS sbSimDriver, SBSHandle AS INTEGER)
	inregs.bx = &H13
	inregs.ax = SBSHandle                   'AX = SBSIM Handle
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	IF NOT (outregs.flags AND 1) THEN        'if no error occured
	   FreeExtMem% = outregs.ax             'return status
	ELSE
	   FreeExtMem% = -1                     'return error condition
	   sbSim.Status = outregs.ax            'status in sbSim.Status
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   GetBufferAddress
'
' Description:  -   This function returns the address and size of the
'                   buffer that has been allocated by SBSIM for a driver.
'
'       Entry:  -   (T) sbSim
'                   (I) Driver (see driver types)
'
'     Returns:  -   (L) Driver Entry Address
'
' Error Codes:  -   (2)
'                   (see SBSIM error codes for description)
'
'************************************************************************
FUNCTION GetBufferAddress! (sbSim AS sbSimDriver, Driver AS INTEGER)
	DIM address AS LONG
	inregs.bx = &H5
	inregs.ax = Driver                      'AX = Selected Driver
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	IF NOT (outregs.flags AND 1) THEN
		address = outregs.dx
		address = address * 65536
		address = address + outregs.ax
		GetBuffInfo! = address              'return Buffer Address
	ELSE
		GetBuffInfo! = -1                   'return error condition
		sbSim.Status = outregs.ax           'status in sbSim.Status
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   GetXMSHandle
'
' Description:  -   This function permits an application to set up it's
'                   own XMS block and relates to obtained XMS handle
'
'       Entry:  -   (I) SBSHandle
'
'     Returns:  -   (I) XMS Handle
'
' Error Codes:  -   (7,12)
'                   (see SBSIM error codes for description)
'
'************************************************************************
FUNCTION GetXMSHandle% (sbSim AS sbSimDriver, SBSHandle AS INTEGER)
	inregs.bx = &H11
	inregs.ax = SBSHandle
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	GetXMSHandle% = outregs.ax
END FUNCTION

'************************************************************************
'
'    Function:  -   InitSBSIM
'
' Description:  -   This function ininializes SBSIM
'
'       Entry:  -   (T) sbSim
'
'     Returns:  -   (I) 0 - SBSIM found (Int Vector set in sbSim.IntVec)
'                      -1 - SBSIM not loaded
'
' Error Codes:  -
'
'************************************************************************
FUNCTION InitSBSIM% (sbSim AS sbSimDriver)
	READ sbSim.IDStr
	READ sbSim.IDOff
	sbSim.IntVec = FindDrv(sbSim.IDStr, sbSim.IDOff)
	IF sbSim.IntVec = -1 THEN
		PRINT "SBSIM driver not loaded";
		InitSBSIM% = -1
	ELSE
		FOR MsgIndex = 0 TO 28
			READ sbSimErrorMsg$(MsgIndex)
		NEXT

		CALL QueryDrivers(sbSim)
		PRINT "Available Drivers: ";
		IF (sbSim.Drivers AND 1) = 1 THEN PRINT "FM ";
		IF (sbSim.Drivers AND 2) = 2 THEN PRINT "DskVoice ";
		IF (sbSim.Drivers AND 4) = 4 THEN PRINT "MemVoice ";
		IF (sbSim.Drivers AND 8) = 8 THEN PRINT "Auxiliary ";
		IF (sbSim.Drivers AND 16) = 16 THEN PRINT "Midi "
		InitSBSIM% = 0
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   LoadExtMem
'
' Description:  -   This function loads a file into extended memory.
'
'       Entry:  -   (I) FileType
'                   (S) Filename
'                   (I) SBSHandle
'
'     Returns:  -   (I) SBSIM Handle
'
' Error Codes:  -   (9,12,13,14)
'                   See sbSimStatus error codes for description
'
'************************************************************************
FUNCTION LoadExtMem% (sbSim AS sbSimDriver, FileType AS INTEGER, Filename AS STRING, SBSHandle AS INTEGER)
	inregs.bx = &H10
	inregs.ax = FileType                    'AX = FileType (0-VOC ONLY)
	inregs.cx = SBSHandle                      'CX = SBSIM Handle
	inregs.ds = VARSEG(Filename)            'DS = FP_Seg of Filename
	inregs.dx = SADD(Filename)              'DX = FP_Off of Filename
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)

	IF (outregs.flags AND 1) = 0 THEN        'if no error occured
	   LoadExtMem% = outregs.ax             'return SBSIM handle
	ELSE
		LoadExtMem% = -1                    'return error condition
		sbSim.Status = outregs.ax           'status in sbSim.Status
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   PauseSound
'
' Description:  -   This register resumes a previously pause sound output
'
'       Entry:  -   (T) sbSim
'                   (I) Driver
'
'     Returns:  -   (none)
'
' Error Codes:  -   (none)
'
'************************************************************************
SUB PauseSound (sbSim AS sbSimDriver, Driver AS INTEGER)
	inregs.bx = (Driver + 1) * 256 + 3
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
END SUB

'************************************************************************
'
'    Function:  -   PlaySnd
'
' Description:  -   This function starts the actual sound output
'
'       Entry:  -   (T) sbSim
'                   (I) Driver
'
'     Returns:  -   (I) sbSimStatus
'
' Error Codes:  -   (5,6,10)
'                   See sbSimStatus error codes for description
'
'************************************************************************
FUNCTION PlaySnd (sbSim AS sbSimDriver, Driver AS INTEGER)
	inregs.bx = (Driver + 1) * 256 + 1
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	IF (outregs.flags AND 1) = 1 THEN
		PlaySnd = -1
		sbSim.Status = outregs.ax
	ELSE
		PlaySnd = outregs.ax
	END IF
END FUNCTION

SUB PrintErr (sbSim AS sbSimDriver)
	PRINT sbSimErrorMsg$(sbSim.Status)
END SUB

'************************************************************************
'
'    Function:  -   QueryDriverEntry
'
' Description:  -   This function returns a long pointer to the entry
'                   address of the specified driver
'
'       Entry:  -   (T) sbSim
'                   (I) Driver (see driver types)
'
'     Returns:  -   (L)(I) sbSimStatus
'
' Error Codes:  -   (2)
'                   (see SBSIM error codes for description)
'
'************************************************************************
FUNCTION QueryDriverEntry! (sbSim AS sbSimDriver, Driver AS INTEGER)
	DIM address AS LONG
	inregs.bx = &H2
	inregs.ax = Driver
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	sbSim.Status = outregs.ax
	IF NOT (outregs.flags AND 1) THEN
	   address = outregs.dx
	   address = address * 256
	   address = address + outregs.ax
	   GetDriverEntry! = address
	ELSE
	   GetDriverEntry! = -1               'return error condition
	   sbSim.Status = outregs.ax
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   QueryDrivers
'
' Description:  -   This function returns a bitmask indicating which
'                   drivers are loaded.
'
'       Entry:  -   (T) sbSimDriver
'
'     Returns:  -   (I) Sets loaded drivers in sbSim.Drivers
'
' Error Codes:  -   (none)
'
'************************************************************************
SUB QueryDrivers (sbSim AS sbSimDriver)
	inregs.bx = &H1
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	sbSim.Drivers = outregs.ax
END SUB

'************************************************************************
'
'    Function:  -   QuerySoundStatus
'
' Description:  -   This function returns a bitmask indicating which
'                   SBSIM sound sources are currently active.
'
'       Entry:  -   (T) sbSim
'
'     Returns:  -   (I) Bitmask indicating currently active drivers
'
' Error Codes:  -   (none)
'
'************************************************************************
SUB QuerySoundStatus (sbSim AS sbSimDriver)
	inregs.bx = &H3
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	sbSim.Active = outregs.ax
END SUB

'************************************************************************
'
'    Function:  -   QueryXMS
'
' Description:  -   This function either determines the availability
'                   of XMS memory, or determines if a specified SBSIM
'                   SBSHandle is free or NOT.
'
'       Entry:  -   (T) sbSim
'                   (I) SBSHandle
'                   if SBSHandle = 0:
'                      Return number of free SBSIM handles
'                   if SBSHandle > 0:
'                      Test if SBSIM handle if free
'
'     Returns:  -   (I) SBSIM Handle Status
'                   if SBSHandle = 0 on entry:
'                       sbSim.Status = 0 - XMS NOT available
'                       sbSim.Status > 0 - Number of free SBSHandles
'                   if SBSHandle > 0 on entry:
'                       sbSim.Status = 0 - SBSHandle is free
'                       sbSim.Status > 0 - Actual XMS handle
'
' Error Codes:  -   (12)
'                   (see SBSIM error codes for description)
'
'************************************************************************
FUNCTION QueryXMS% (sbSim AS sbSimDriver, SBSHandle AS INTEGER)
	inregs.bx = &H4
	inregs.ax = SBSHandle
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	QueryXMS% = outregs.ax
END FUNCTION

'************************************************************************
'
'    Function:  -   ReadSoundStatus
'
' Description:  -   This function returns the value of the sound
'                   status word.
'
'       Entry:  -   (T) sbSim
'                   (I) Driver
'
'     Returns:  -
'
' Error Codes:  -   (none)
'
'************************************************************************
FUNCTION ReadSoundStatus% (sbSim AS sbSimDriver, Driver AS INTEGER)
	inregs.bx = (Driver + 1) * 256 + 5
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	ReadSoundStatus% = outregs.ax
END FUNCTION

'************************************************************************
'
'    Function:  -   ResumeSound
'
' Description:  -   This register resumes a previously pause sound output
'
'       Entry:  -   (T) sbSim
'                   (I) Driver
'
'     Returns:  -   (none)
'
' Error Codes:  -   (none)
'
'************************************************************************
SUB ResumeSound (sbSim AS sbSimDriver, Driver AS INTEGER)
	inregs.bx = (Driver + 1) * 256 + 4
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
END SUB

'************************************************************************
'
'    Function:  -   SetBufferAddress
'
' Description:  -   This function sets the address and size of a buffer
'                   for a driver.
'
'       Entry:  -   (T) sbSim
'                   (I) Driver - (See driver types)
'                   (L) BufferAddress
'                   (I) BufferSize
'
'     Returns:  -   (I) sbSimStatus
'
' Error Codes:  -   (2)
'                   (see SBSIM error codes for description)
'
'************************************************************************
FUNCTION SetBufferAddress% (sbSim AS sbSimDriver, Driver AS INTEGER, BufferAddress AS LONG, BufferSize AS INTEGER)
	inregs.bx = &H6
	inregs.ax = Driver                      'AX = Selected Driver
	inregs.ds = hiword%(BufferAddress)      'DS = FP_Seg of Buffer
	inregs.dx = loword%(BufferAddress)      'DX = FP_Off of Buffer
	inregs.cx = BufferSize                  'CX = Buffer Size
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	IF NOT (outregs.flags AND 1) THEN
		SetBufferAddress% = outregs.ax      '
	ELSE
		SetBufferAddress% = -1              'return error condition
		sbSim.Status = outregs.ax           'status in sbSim.Status
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   SetXMSHandle
'
' Description:  -   This function allows an application program to setup
'                   it's own XMS block, and to set the obtained XMS handle
'                   into an SBSIM handle.
'
'       Entry:  -   (I) SBSHandle (0 - use first free handle)
'                   (I) XMSHandle
'
'     Returns:  -   (I) SBSHandle
'
' Error Codes:  -   (7,12,13)
'                   (see SBSIM error codes for description)
'
'************************************************************************
FUNCTION SetXMSHandle% (sbSim AS sbSimDriver, SBSHandle AS INTEGER, XMSHandle AS INTEGER)
	inregs.bx = &H12
	inregs.ax = SBSHandle                      'AX = SBSIM Handle
	inregs.dx = XMSHandle                      'DX = XMS Handle
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	IF NOT (outregs.flags AND 1) THEN           'if NOT error occured
	   SetXMSHandle% = outregs.ax              'return SBSIM Handle
	ELSE
	   SetXMSHandle% = -1                      'return error condition
	   sbSim.Status = outregs.ax               'status in sbSim.Status
	END IF
END FUNCTION

'************************************************************************
'
'    Function:  -   StartSnd
'
' Description:  -   This driver sets up the specified driver for a
'                   subsequent sound output.
'
'       Entry:  -   (T) sbSim
'                   (I) Driver (see driver types)
'                   (S) Filename
'                   (L) BufferAddress
'
'     Returns:  -    0 = No error
'                   -1 = error condition in sbSim.Status
'
' Error Codes:  -   (5,6,7,8,9)
'                   (see SBSIM error codes for description)
'
'************************************************************************
FUNCTION StartSnd (sbSim AS sbSimDriver, Driver AS INTEGER, Filename AS STRING, BuffSeg AS INTEGER, SBSHandle AS INTEGER)
	inregs.bx = (Driver + 1) * 256
	inregs.cx = 0
	IF Driver = 2 THEN
		IF SBSHandle > 0 THEN
			inregs.dx = 0                   'DX = 0 for SBSIM handle
			inregs.ax = SBSHandle           'AX = SBSIM Handle
		ELSE
			inregs.dx = BuffSeg
            inregs.ax = &h1a
		END IF
	ELSE
		inregs.ds = VARSEG(Filename)        'DS = FP_Seg of filename
		inregs.ax = SADD(Filename)          'AX = FP_Off of filename
	END IF
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	IF (outregs.flags AND 1) THEN        'if no error occured
	   StartSound% = outregs.ax             'return status
	ELSE
	   StartSound% = -1                     'return error condition
	   sbSim.Status = outregs.ax            'status in sbSim.Status
	END IF
END FUNCTION

SUB StopSound (sbSim AS sbSimDriver, Driver AS INTEGER)
	inregs.bx = ((Driver + 1) * 256) + 2
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
END SUB

'************************************************************************
'
'    Function:  -   Version
'
' Description:  -   This function returns the version number of SBSIM
'
'       Entry:  -   (T) sbSimDriver
'
'     Returns:  -   (I) Sets the version in the sbSim.Version
'
' Error Codes:  -   (None)
'
'************************************************************************
SUB Version (sbSim AS sbSimDriver)
	inregs.bx = &H0
	CALL INTERRUPTX(sbSim.IntVec, inregs, outregs)
	sbSim.Version = outregs.ax
END SUB

'************************************************************************
'
'    Function:  -   LoadMem%
'
' Description:  -   This function loads a file into conventional memory.
'
'       Entry:  -   (S) Filename
'
'     Returns:  -   (L) Segment of loaded file.
'
' Error Codes:  -   (9,12,13,14)
'                   See sbSimStatus error codes for description
'
'************************************************************************
FUNCTION LoadMem% (lpszFileName AS STRING)
    DIM FSize AS INTEGER
    DIM FHandle AS INTEGER
    DIM BuffSeg AS INTEGER
    DIM BytesRead

    LoadMem% = -1

    FSize = FileSize%(lpszFileName)

PRINT "FileSize:";hex$(FSize)

    IF NOT (FSize = -1) THEN
       BuffSeg = AllocMem%( INT( FSize / 16 ) + 1 )
       IF NOT (BuffSeg = -1) THEN
           FHandle = OpenFile%(lpszFileName)

PRINT "File Handle:";hex$(FHandle)

           IF NOT (FHandle = -1) THEN
               BytesRead = ReadFile%(FHandle, BuffSeg, FSize)

PRINT "Bytes Read:";hex$(BytesRead)

               IF NOT (BytesRead = -1) THEN
                   LoadMem% = BuffSeg
               END IF
               DOSError = CloseFile%(FHandle)
           END IF
       END IF
   END IF
END FUNCTION

