/*********************************************************************
*
*  PROGRAM NAME: PLAYCMF.C
*
*  COMPILER: Borland/Turbo C/C++
*
*  DESCRIPTION: Plays a .CMF FM file from the disk using the FMDRV
*               driver (accessed from SBSIM).  To run the program,
*               type the following at the command line:
*
*               PLAYCMF FILENAME
*
*               Where FILENAME is the drive/path/filename of the .CMF
*               file to play.
*
*  NOTE: SBSIM driver must be loaded before running this program.
*
*********************************************************************/
#define FM_DRIVER 0x01

#include <ctype.h>
#include <conio.h>
#include <fcntl.h>
#include <io.h>
#include <stdio.h>
#include "drvrfunc.c"
#include "drvrfunc.h"


/********************************************************************/
void main(int argc, char **argv)
{
  char Command,
       UserQuit;

  int    FileHandle;
  SIMERR RetValue;


  if (argc != 2)  // argc = no. of parameters entered on command line
  {
    puts("Command line must contain EXACTLY ONE filename parameter!");
    return;  // Terminate program
  }


  /*--- SEE IF SBSIM DRIVER HAS LOADED -------*/
  SIMint = FindDvr("SBSIM", 0x0103);  // Get SBSIM's interrupt no.
  if (SIMint == 0)
  {
    puts("SBSIM driver not loaded!");
    return;  // Terminate program
  }

  /*--- SEE IF FMDRV DRIVER HAS LOADED -------*/
  if (!(GetDrvrs() & FM_DRIVER))
  {
    puts("FMDRV not loaded!");
    return;  // Terminate program
  }


  /*--- LOAD THE FILE ENTERED ON COMMAND LINE (stored in argv[1] ----*/
  if ((FileHandle = _open(argv[1], O_BINARY | O_RDONLY)) == -1)
  {
    printf("FILE: %s not successfully opened!\n", argv[1]);
    return;  // Terminate program
  }


  /*--- StartSnd() LOADS THE FILE AND INITIALIZES THE DRIVER ---------*/
  RetValue = StartSnd(FM, (void far *) argv[1], NULL, NULL);
  if (RetValue != SIMerr_NoErr)
  {
    printf("ERROR CALLING SBSIM: %s\n", errorMsg[RetValue]);
    _close(FileHandle);
    return;  // Terminate program
  }

  /* PlaySnd() BEGINS PLAYING THE FILE -------------------------------*/
  RetValue = PlaySnd(FM);
  if (RetValue != SIMerr_NoErr)
  {
    printf("ERROR CALLING SBSIM: %s\n", errorMsg[RetValue]);
    _close(FileHandle);
    return;  // Terminate program
  }


  clrscr();  // Clear screen
  printf("\n\n\n\n\n     SELECT ONE OF THE FOLLOWING OR WAIT UNTIL DONE:\n\n");
  printf("                   (P)ause\n");
  printf("                   (R)esume\n");
  printf("                   (Q)uit\n");

  UserQuit = FALSE;

  /*--- PROCESS USER INTERACTION OR WAIT FOR SOUND TO STOP -----------*/
  do
  {
    if (kbhit())  // Was a key pressed?
    {
      Command = toupper(getch());  // Get char that's in buffer
      if (Command == 'P')
	PauseSnd(FM);
      else if (Command == 'R')
	ResumeSnd(FM);
      else if (Command == 'Q')
	UserQuit = TRUE;
    }
    // Exit do-while loop if file is done playing or user typed 'Q'
  } while (GetSndStat(FM) != 0 && UserQuit == FALSE);


  /*--- IF SOUND IS STILL PLAYING AND USER HIT 'Q', STOP THE SOUND ---*/
  if (GetSndStat(FM) != 0 && UserQuit == TRUE)
    StopSnd(FM);

  return;
}