(*********************************************************************
*
*  PROGRAM NAME: PLAYMIDI.PAS
*
*  COMPILER:     Borland Turbo Pascal ver. 4 or later
*
*  DESCRIPTION: Plays a .MID MIDI file from the disk using the SBMIDI
*               driver (accessed from SBSIM).  To run the program,
*               type the following at the command line:
*
*               PLAYMIDI FILENAME
*
*               Where FILENAME is the drive/path/filename of the .MID
*               file to play.
*
*  NOTE: SBSIM driver must be loaded before running this program.
*
*********************************************************************)

program playmidi;
uses dos,crt;

const MIDI_DRIVER = $10;



{$I drvrfunc.pas}


var Command:char;
    UserQuit:boolean;
    RetValue:simerr;
    FileName:string;
    FileHandle:integer;

(********************************************************************)
begin (* main *)

if paramcount <> 1  (* no. of parameters entered on command line *)
then begin
     writeln('Command line must contain EXACTLY ONE filename parameter!');
     halt;   (* Terminate program *)
     end;

(*--- SEE IF SBSIM DRIVER HAS LOADED --*)
SIMint := FindDvr('SBSIM', $0103);  (* Get SBSIM's interrupt no. *)
if SIMint = 0
then begin
     writeln('SBSIM driver not loaded!');
     halt;   (* Terminate program *)
     end;

(*--- SEE IF MIDI DRIVER HAS LOADED ---*)
if (GetDrvrs and MIDI_DRIVER) <> MIDI_DRIVER
then begin
     writeln('SBMIDI not loaded!');
     halt;
     end;

(*--- LOAD THE FILE -------------------*)
FileHandle := DosOpen(paramstr(1), ReadAccess);
if (FileHandle = -1)
then begin
     writeln('FILE: ',paramstr(1), ' not successfully opened!');
     halt;    (* Terminate program *)
     end;

DosClose(FileHandle);

FileName:=paramstr(1);  (* copy to string with more space before modifying *)

(*--- StartSnd() LOADS THE FILE AND INITIALIZES THE SBMIDI DRIVER ---*)
RetValue := StartSnd(Midi, AsciiZ(FileName), false, 0);
if RetValue <> SIMerr_NoErr
then begin
     writeln('ERROR CALLING SBSIM: ', errorMsg[RetValue]);
     halt;  (* Terminate program *)
     end;

(*--- PlaySnd() BEGINS PLAYING THE FILE -------------*)
RetValue := PlaySnd(Midi);
if RetValue <> SIMerr_NoErr
then begin
     writeln('ERROR CALLING SBSIM: ', errorMsg[RetValue]);
     halt;  (* Terminate program *)
     end;

clrscr;  (* Clear screen *)
writeln(#10#10#10#10#10'     SELECT ONE OF THE FOLLOWING OR WAIT UNTIL DONE:');
writeln('                   (P)ause');
writeln('                   (R)esume');
writeln('                   (Q)uit');

UserQuit := false;

(*--- PROCESS USER INTERACTION OR WAIT FOR SOUND TO STOP -----------*)
repeat
     if (keypressed)  (* Was a key pressed? *)
     then begin
          Command := upcase(readkey);  (* Get char that's in buffer *)
          if Command = 'P'
          then PauseSnd(Midi)
          else if Command = 'R'
          then ResumeSnd(Midi)
          else if Command = 'Q'
          then UserQuit := true;
          end;
(* Exit do-while loop if file is done playing or user typed 'Q' *)
until (UserQuit = true) or (GetSndStat(Midi) = 0);


(*--- IF SOUND IS STILL PLAYING AND USER HIT 'Q', STOP THE SOUND ---*)
if (GetSndStat(Midi) <> 0) and (UserQuit = true)
then StopSnd(Midi);

end.